#ifndef SURFACE_H
#define SURFACE_H

#include "text.h"
#include <png.h>
#include <cstdio>
#include <windows.h>


const int SURFACE_WIDTH              = 48;  // SVG̕
const int SURFACE_HEIGHT             = 64;  // SVG̍
const unsigned int SCALE_LOWER_LIMIT = 25;  // TCY̔{(%)̉
const unsigned int SCALE_UPPER_LIMIT = 300; // TCY̔{(%)̏


class SurfaceCache{
  int width;
  int height;
  FILE* fp;                   // LbVΏۂ̉摜t@C
  BITMAPINFO bitmapInfo;      // rbg}bvɂۂ̏(݂̓tJ[24bitŌŒ)
  png_structp pPng;
  png_infop pngInfo;
  BYTE** image;               // sNZf[^
  int bottom;                 // ɓǂݍ񂾃sNZ̉[AǂݍłȂꍇ-1Ƃ邱
                              // (摜[1s܂œǂݍ񂾎ɂāA̓ǂݍ݂ŉ摜[Jn_Ƃ)
  
  bool SurfaceCache::ReadPngInfo();
  bool ReadPngPixel(int valBottom);
public:
  void Clear();               // LbVɂ
  // 摜ǂݍޏ
  // C^[X̏ꍇ͂1png_read_rowsĂяoƓǂݍ݂ԂɂĂ
  bool ReadInfo(const char* surfaceFile);
  bool WriteBuffer(BYTE* bmpBuffer, int spriteID);
  const BITMAPINFO* GetBITMAPINFO() const{ return &bitmapInfo; }
  
  SurfaceCache();
  
  ~SurfaceCache(){
    Clear();
  }
  
};



class Surface{
  AutoString surfaceFile;                 // T[tBX摜t@C̃tpX
  static const Surface* prevSurface;      // LbVĂT[tBX
  static SurfaceCache cache;
  static int scaledWidth;
  static int scaledHeight;
  static bool surfaceNumViewMode;         // T[tBXԍ\Lǂ
  static bool useScaling;                 // TCYLǂ
  static int fontSize;                    // tHg̃TCY
  
public:
  // T[tBX摜t@C̃pXݒ
  void SetSurfaceFile (const char* path){
    delete[] surfaceFile.str;             // Âf[^Ȃ폜
    surfaceFile.str = NewString(path);    // T[tBX摜t@C̃pXo^
  }
  
  // T[tBX摜t@C̃pX
  const char* GetSurfaceFile() const{ return surfaceFile.str; }
  
  int WriteImage(int surfaceID, int spriteID, HBITMAP hBitmap) const;
  static void ClearCache(){ cache.Clear(); }
  static int GetWidth(){ return (useScaling) ? (scaledWidth) : (SURFACE_WIDTH); }
  static int GetHeight(){ return (useScaling) ? (scaledHeight) : (SURFACE_HEIGHT); }
  static unsigned int SetScale(const char* percent);
  static bool GetSurfaceNumViewMode(){ return surfaceNumViewMode; }
  static void SetSurfaceNumViewMode(bool valMode){ surfaceNumViewMode = valMode; }
  static bool GetScalingMode(){ return useScaling; }
  static void SetScalingMode(bool valMode){ useScaling = valMode; }
};

void InitializeBITMAPINFO(BITMAPINFO& bitmapInfo, int width, int height, unsigned short bitCount);


#endif // SURFACE_H

